C*****************************************
       program Kernel Density Approximation Graph 
C*****************************************
       implicit none
	   include 'common.fi'

       write(6,*) '================================================='
	   write(6,*) ' PROGRAM IS USED TO PLOT GRAIN BOUNDARY PROPERTY '
	   write(6,*) ' DISTRIBUTIONS FROM LISTS OF GRAIN BOUNDARY '
	   write(6,*) ' TRIANGLES USING THE KERNEL DENISTY APPROXIMATION '
	   write(6,*) ' AS DESCRIBED BY GLOWINSKI AND MORAWEIC, 2014 '
	   write(6,*) ' http://dx.doi.org/10.1007/s11661-014-2325-y'
	   write(6,*) ' '
	   write(6,*) ' rohrer@cmu.edu'
	   write(6,*) ' version 02/17/2021'
       write(6,*) '================================================='
	   
	   version = 'version 02/17/2021'
       ! version 05/24/2019: added curvature distributions
       ! version 06/10/2019: fixed small error in area summation and
       ! version 06/10/2019: added normalization for symmetry
       ! version 06/10/2019: added status update
       ! version 06/11/2019: read grid points from a list
       ! version 12/21/2019: this was modified to deal with velocity
       ! version 04/02/2020: modified to create projections in [001],
       ! version 04/02/2020: [110], [111], and [100]
       ! version 04/02/2020: improved suggestions for the GMT command
       ! version 08/07/2020: fixed acos to never return nan
       ! verison 08/08/2020: modified to count unique triangles
       ! version 08/09/2020: modified to write a list of triangles filtered by misorientation
       ! version 02/17/2021: fixed a bug in the acos check

       !some constants that might be useful
       pi = 4.0*atan(1.0)
       !some formating statements used to format typed output
  5    format(A,A)
  6    format(A)
  7    format(A,I4,A,I4)
  14   format(A,I2,2X,A,I2)
  15   format(A,I12,A)
  16   format(A,I8,A,I9,A)
  20   format(A,A,A,A,f5.2,1x,f8.2,1x,f7.2,A)
  21   format(A,3I3,A,f5.2)
  22   format(A,A,A,A,f6.3,1x,f8.3,1x,f8.3)
  24   format(A,4(1x,f4.1))
  25   format(A,4(1x,f5.3))
  26   format(A,A,A,f4.2,1x,f4.2,1x,f4.2,A)
  27   format(14f10.4)
  28   format(12f8.4)


       open(21, file='input.txt', status='old') ! Open the file with the program
                                                ! parameters.
	   read(21,*) keyword                       ! This reads the filename with the data
       read(21,*) mode                          ! Specifies the type of output (2D or 5D)
	   read(21,*) tempword, tempword            ! Reads a line containing headers
       read(21,*) msym, radian, dist_type       ! Specifies the symmetry, units for angles, plot type
       read(21,*) tempword                      ! Read a header line
	   read(21,*) rho_m, rho_p, BallVolumeIndex ! Read the dimensions of the aperture
       read(21,*) ax(1), ax(2), ax(3), ang      ! This is the fixed misorientation for the 5D mode
       read(21,*) projection                    ! projection axis for output
       read(21,*) name                          ! added to the output filename
       read(21,*) errors                        ! determines whether or not error estimates are written
       read(21,*) numDistinctGBs                ! number of distinct boundaries
       read(21,*) list_out                      ! flag for exporting list of tiangles filtered my misorientation
       read(21,*) filter_only                   ! flag for skipping the graphing part
       close(21)                                ! Close the input file

       !You need to set this path correctly for it to run
       pathname='/Users/gr20/Desktop/210217_new_velocity/' !don't know how to read strings with '/'
       DataName=trim(pathname)//'KDA_graph/data_in/'//keyword  !setup the path for the source of data
       OutPath=trim(pathname)//'KDA_graph/results_out/'  !setup the path for the results

       point = index(keyword,'.') - 1

       !this updates the user about the parameters on the consol.
       write(6,5)'I am going to look for data in the file labeled: ', keyword
       write(6,14)'msym =',msym,'radian =',radian
       if (dist_type.eq.1) then
        write(6,6)'the input is being interpreted as grain boundary area data'
       endif
       if (dist_type.eq.2) then
        write(6,6)'the input is being interpreted as grain boundary energy data'
       endif
       if (dist_type.eq.3) then
        write(6,6)'the input is being interpreted as grain boundary curvature data'
       endif
       if (dist_type.eq.4) then
        write(6,6)'the input is being interpreted as grain boundary velocity data'
       endif
       if (dist_type.eq.5) then
        write(6,6)'the input is being interpreted as grain boundary velocity x curvature data'
       endif

       !write(6,6)'Note: if there are more and 10 million representations'
       !write(6,6)'of triangles within the aperture, the program will crash.'
       !write(6,6)'if this is the problem, the fix is to increase the dimensions'
       !write(6,6)'of ftrd(10000000,7) in the common, and recompile with make.'

       !this is the section where we set the viewpoint for the projection
       !for a [001] projection, there is no rotation of the data
	   if (projection.eq.1) then
		   ax_ref(1)=0.0
	       ax_ref(2)=0.0
	       ax_ref(3)=1.0
	       ang_ref= 0.0
	       write(6,6)'The [001] direction will be normal to the plane of the drawing.'	   
	   endif
       !for a [110] projection, rotate 90 deg around [1-10]
	   if (projection.eq.2) then
	       ax_ref(1)=0.7071
	       ax_ref(2)=-0.7071
	       ax_ref(3)=0.0
           ang_ref= pi*(90.0/180.0)
	       write(6,6)'The [110] direction will be normal to the plane of the drawing.'			   
	   endif
       !for a [111] projection, rotate 54.7 deg around [1-10]
	   if (projection.eq.3) then
	       ax_ref(1)=0.7071
	       ax_ref(2)=-0.7071
	       ax_ref(3)=0.0
           ang_ref= pi*(54.7/180.0)
	       write(6,6)'The [111] direction will be normal to the plane of the drawing.'			   
	   endif
       !for a [100] projection, rotate 90 deg around [0-10]
	   if (projection.eq.4) then
	       ax_ref(1)=0.0
	       ax_ref(2)=-1.0
	       ax_ref(3)=0.0
           ang_ref= pi*(90.0/180.0)
	       write(6,6)'The [100] direction will be normal to the plane of the drawing.'			   
	   endif	
	   write(6,*) ' '
       !this is the matrix that will rotate the vectors on the plot.
	   call AAToG (ax_ref, ang_ref, g_ref)
	   call trans (g_ref, g_ref_t)

       ! This part determines the total number of lines
       ! in the data file and assigns it to the variable, nnline
		nnline = 0 ! Initialize counter
        open(41, file=DataName, status='old')
 60     continue
		read(41,*,end=61) ! read line, if end of file, goto 61
		nnline=nnline+1  ! Increment counter
		goto 60
 61     continue
        close(41)  ! we are finished, close the file, report result.
		write(6,15) 'There are  ',nnline,' lines in the data file.'
		
        ! Check each line for the string '#'. If found, we know this line
        ! is part of the header.  If not, we have reached the data section.
        open(41, file=DataName, status='old') ! open the file

        header = 0 ! Initialize counter
        hash = 0   ! Initialize flag
	    do i=1,nnline  ! loop over each line in the file
	     read(41, "(a)") inline    ! reads line into a string
	     hash = index(inline, '#') ! if the character "#" in the string, hash>1.
	     if (hash.ne.0) then       ! check the flag
	      header=header+1          ! when "#" found, it is a header line
		  hash=0                   ! reset the flag
	     else
		  goto 80                  ! when hash is = 0, we have reached the end of the header
	     endif
	    enddo  ! closes i=1,nnline
 80    close(41)				   ! close the file, report the results
	   write(6,15)'There are ',header,' lines in the header'
       write(6,15) 'There are  ',nnline-header,' triangles in the data file.'

       !the following are normalization constants for grain boundary plane distribution data
       BallVolume(1) = 0.0000641361 !3° mis, 7° plane
       BallVolume(2) = 0.000139158  !5° mis, 5° plane
       BallVolume(3) = 0.000287439  !5° mis, 7° plane
       BallVolume(4) = 0.00038019   !5° mis, 8° plane
       BallVolume(5) = 0.000484151  !6° mis, 7° plane
       BallVolume(6) = 0.000747069  !7° mis, 7° plane
       BallVolume(7) = 0.00145491   !8° mis, 8° plane

       !This opens the output file, where results will be written
       !open(30, file=OutPath//adjustl(keyword(1:point))//name//'_gmt1.dat',status='unknown')
       open(30, file=keyword(1:point)//name//'_gmt1.dat',status='unknown')
       write(30,"(4(1x,f4.1))")  ax(1), ax(2), ax(3), ang
       write(6,21) 'misorientation = [',int(ax(1)), int(ax(2)), int(ax(3)),'] ', ang
       !if data on errors was requested, a file is opened for this distribution
       if (errors.eq.1) then
        !open(31, file=OutPath//trim(keyword(1:point))//name//'_errors_gmt1.dat',status='unknown')
        open(31, file=keyword(1:point)//name//'_errors_gmt1.dat',status='unknown')
        write(31,"(4(1x,f4.1))")  ax(1), ax(2), ax(3), ang
       else
        continue
       endif
       !If the list_out options is selected, a file is opened
       if (list_out.eq.1) then
        !open(32, file=OutPath//trim(keyword(1:point))//name//'_list.txt',status='unknown')
        open(32, file=keyword(1:point)//name//'_list.txt',status='unknown')
       else
        continue
       endif

       call get_symop (msym, O, nsymm)
       !call get_symop (msym, O, nsymm) calls the subroutine that contains the symmetry operators.
       !The variable msym, read from the input file, specifies the symmetry (1 = tetragonal,
       !2 = hexagonal, 3 = cubic, 4 = trigonal, 5 = orthorhombic), O is a matrix containing the
       !operators, and nsymm is the number of symmetry operators.  The routine returns O and nsymm

       !here we have to adjust the normalization factors to account for symmetry
       do i=1,7
        BallVolume(i)=BallVolume(i)*(float(nsymm)/24.0)
       enddo

       rho_m=rho_m*(pi/180.0)  !We have to convert the aperture values to radians
       rho_p=rho_p*(pi/180.0)
       rho_pSq=rho_p*rho_p     !compute rho_p squared - a constant used later

       !we convert the axis angle description of the misorientation to a 3x3 orientation matrix, gFixed.
       !we start by normalizing the axis.
       len=sqrt(ax(1)*ax(1)+ax(2)*ax(2)+ax(3)*ax(3))
       ax(1)=ax(1)/len
       ax(2)=ax(2)/len
       ax(3)=ax(3)/len
       ang=ang*(pi/180.0)           !convert the misorientation angle to radians
       write(6,25) 'misorientation = ',ax(1), ax(2), ax(3), ang
       call AAToG (ax, ang, gFixed) !convert the axis and angle to an orientation matrix, g.
       call trans (gFixed, gFixedT) !get the transpose

       !read in the sampling points
       open(22, file='grid_points.txt',status='old')
       read(22,*)num_samplPts
       do i=1,num_samplPts
        read(22,*)samplPts(1,i),samplPts(2,i),samplPts(3,i)
       enddo
       close(22)

       do i=1,cnt
        dist(i)=0.0
        err(i)=0.0
        num(i)=0.0
       enddo

       !In the next section of code, we read the triangles in one at a time.
       !If they are within the aperture, we save their information.
       open(41, file=DataName, status='old') ! open the file and read the header
       do i=1,header
        read(41, "(a)") inline ! read a line of the header
       enddo! ends i=1,header loop
       !ct1=0
       !ct2=0
       totalArea = 0
       ct = 0 !introduce a counter for triangles
       num_unique =0 !introduce a counter for unique triangles
       do i=header+1,nnline !start on the line after the header, continue to end.
        area_real = 0.0  !provide zeros for all values that might be read
        energy = 0.0
        curv = 0.0
        vel = 0.0
        velxcurve = 0.0
        if (dist_type.eq.1) then !This case is for a GB plane distribution
         read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real
         area=area_real
        endif

        if (dist_type.eq.2) then !This case is for a GB energy distribution
         read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), polar, azimuth, area_real, energy
         polar=polar*(pi/180.0)
         azimuth=azimuth*(pi/180.0)
         area=abs(energy)
         call AnglesToV (polar, azimuth, normal_lab)
        endif

        if (dist_type.eq.3) then !This case is for a GB curvature distribution
         read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, curv
         area=abs(curv)
        endif

        if (dist_type.eq.4) then !This case is for a GB velocity distribution
         read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, curv, vel
        area=abs(vel)
        endif

        if (dist_type.eq.5) then !This case is for a GB velocity * curvature distribution
         read(41,*) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, curv, vel, velxcurve
         area=abs(velxcurve)
        endif

        ! outputs from some programs produce zero Euler angles when
        ! when one or both orientiations are not known.  These lines are
        ! ignored because they will cause problems later.
        !if (e1(1).eq.0.00.and.e1(2).eq.0.00.and.e1(3).eq.0.00) then
        ! goto 1000 ! Sends it to the next line
        !else
        ! continue
        !endif
        !if (e2(1).eq.0.00.and.e2(2).eq.0.00.and.e2(3).eq.0.00) then
        ! goto 1000 ! Sends it to the next line
        !else
        ! continue
        ! endif

        !a section to condition the data
        !open(42, file='cnd.txt', status='unknown')
        !if (abs(curv).gt.3.0) then
         !ct1=ct1+1
         !goto 1000 ! Sends it to the next line
        !else
         !write(42,28) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, curv, vel
         !ct2=ct2+1
         !goto 1000
        !endif




        totalArea=totalArea+area !if a good triangle, add the area

        !If the Euler angles are provided in units of degrees, these
        !lines convert them to radians
        if (radian.eq.0) then
         e1(1)=e1(1)*(pi/180.0)
         e1(2)=e1(2)*(pi/180.0)
         e1(3)=e1(3)*(pi/180.0)
         e2(1)=e2(1)*(pi/180.0)
         e2(2)=e2(2)*(pi/180.0)
         e2(3)=e2(3)*(pi/180.0)
        else
         continue
        endif

        ! The Euler angles are convert to 3x3 orientation matrices, g_o1 and g_o2.
        call EToG(e1,g_o1)
        call EToG(e2,g_o2)

        ! Next, we compute delta_m and for all symm equivalents of this
        ! boundary and compare it to the aperture, rho_m.  If it is smaller than
        ! the aperture, we find the normal vectors and record them, with the
        ! area, in the arrary fltd.  This is repeated for every triangle until
        ! the end of the file.
          in_aperture=0
		  do i_sy=1,nsymm             ! loop over all symmetry elements
           call symop(O,i_sy,so1)     ! get first symmetry element for inner loop (so1)
           call MToM (so1,g_o1,gg1)	  ! hit first orientation with a symmetry element
           call MToV(gg1, normal_lab, normal_grain1)
           do j_sy=1,nsymm             ! loop over all symmetry elements
		    call symop(O,j_sy,so2)     ! get the second symmetry element for outer loop (so2)
		    call MToM(so2,g_o2,gg2)	   ! hit second orientation with a symmetry element
		    call trans(gg2,g2sT)       ! take the transpose of the second orientation
		    call MToM(gg1,g2sT,dg)     ! calculate the misorientation (dg)
            call trans(dg,dgT)
            call MToM(dg,gFixedT,diffFromFixed)
            tem=(diffFromFixed(1,1)+diffFromFixed(2,2)+diffFromFixed(3,3)-1)/2
            if (tem.gt.1.0) tem=1.0
            if (tem.lt.-1.0) tem=-1.0
            delta_m=acos(tem)
            if (delta_m.lt.rho_m) then  ! check to see if it is within the aperture
             ct=ct+1 !increment ct everytime a triangle is within the aperture
             in_aperture=in_aperture+1
             call MToV(dgT, normal_grain1, normal_grain2)
             ftrd(ct,1)=normal_grain1(1)
             ftrd(ct,2)=normal_grain1(2)
             ftrd(ct,3)=normal_grain1(3)
             ftrd(ct,4)=-normal_grain2(1)
             ftrd(ct,5)=-normal_grain2(2)
             ftrd(ct,6)=-normal_grain2(3)
             ftrd(ct,7)=area
            else
             continue
            endif
            !call MToM(dgT,gFixedT,diffFromFixed)
            call MToM(gFixed,dgT,diffFromFixed)
            tem=(diffFromFixed(1,1)+diffFromFixed(2,2)+diffFromFixed(3,3)-1)/2
            if (tem.gt.1.0) tem=1.0
            if (tem.lt.-1.0) tem=-1.0
            delta_m=acos(tem)
            if (delta_m.lt.rho_m) then  ! check to see if it is within the aperture
             ct=ct+1 !increment ct everytime a triangle is within the aperture
             in_aperture=in_aperture+1
             call MToV(dgT, normal_grain1, normal_grain2)
             ftrd(ct,1)=-normal_grain2(1)
             ftrd(ct,2)=-normal_grain2(2)
             ftrd(ct,3)=-normal_grain2(3)
             ftrd(ct,4)=normal_grain1(1)
             ftrd(ct,5)=normal_grain1(2)
             ftrd(ct,6)=normal_grain1(3)
             ftrd(ct,7)=area
            else
             continue
            endif
		   enddo ! closes j_sy=1,nsymm
		  enddo ! closes i_sy=1,nsymm
          !this is a progress update
		  if (mod(i,250000).eq.0) then
	       write(6,16) 'classified ',i, ' of ',nnline,' triangles'
	      endif

          if (in_aperture.gt.0.) then
           num_unique=num_unique+1
           if (list_out.gt.0.) then
            !write(32,27) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, energy, curv, vel, velxcurve
            write(32,28) e1(1), e1(2), e1(3), e2(1), e2(2), e2(3), normal_lab(1), normal_lab(2), normal_lab(3), area_real, curv, vel

           else
            continue
           endif
          else
           continue
          endif
 1000     continue ! so we go to the next line of the data file and repeat
         enddo    ! This closes i=header+1,nnline, the loop over each line of data.
         !goto 9000
         write(6,15)'there are ',ct,' triangles within the misorientation aperture'
         write(6,15)'there are ',num_unique,' unique triangles within the misorientation aperture'
         close(41)

         if (filter_only.eq.1) then
          goto 9000
         endif

         do i=1,num_samplPts
          fixedNormal1_pre(1)=samplPts(1,i)
          fixedNormal1_pre(2)=samplPts(2,i)
          fixedNormal1_pre(3)=samplPts(3,i)
          call MToV (g_ref_t, fixedNormal1_pre, fixedNormal1)!change projection
          call MToV(gFixedT, fixedNormal1, fixedNormal2)
          sum = 0.0
          numTris = 0
          do j=1,ct
           normal_grain1(1)=ftrd(j,1)
           normal_grain1(2)=ftrd(j,2)
           normal_grain1(3)=ftrd(j,3)
           normal_grain2(1)=ftrd(j,4)
           normal_grain2(2)=ftrd(j,5)
           normal_grain2(3)=ftrd(j,6)
           area=ftrd(j,7)
           sign=1.0
           chi1=acos(sign*( (normal_grain1(1)*fixedNormal1(1)) + (normal_grain1(2)*fixedNormal1(2)) + (normal_grain1(3)*fixedNormal1(3)) ))
           chi2=acos(-sign*( (normal_grain2(1)*fixedNormal2(1)) + (normal_grain2(2)*fixedNormal2(2)) + (normal_grain2(3)*fixedNormal2(3)) ))
           delta_pSq=0.5*((chi1*chi1)+(chi2*chi2))
           if (delta_pSq.lt.rho_pSq) then
            numTris=numTris+1
            sum = sum + area
           else
            continue
           endif
           sign=-1.0
           chi1=acos(sign*( (normal_grain1(1)*fixedNormal1(1)) + (normal_grain1(2)*fixedNormal1(2)) + (normal_grain1(3)*fixedNormal1(3)) ))
           chi2=acos(-sign*( (normal_grain2(1)*fixedNormal2(1)) + (normal_grain2(2)*fixedNormal2(2)) + (normal_grain2(3)*fixedNormal2(3)) ))
           delta_pSq=0.5*((chi1*chi1)+(chi2*chi2))
           if (delta_pSq.lt.rho_pSq) then
            numTris=numTris+1
            sum = sum + area
           else
            continue
           endif
          enddo !j loop over triangles
            dist(i)=sum ! assigns the total area at this sample point
            num(i)=numTris ! assigns number of energy values added at this sample point
            if (mod(i,500).eq.0) then
             write(6,7) 'At grid point ',i,' of ',num_samplPts
            endif

         enddo !i loop over sample points

         dist_max=0.0
         dist_min=1000.0

         !normalize the distributions, and find find the extreme values
         if (dist_type.eq.1) then !This part is for grain boundary plane distributions
          do i=1, num_samplPts
           dist(i)=dist(i)/totalArea
           dist(i) = dist(i)/BallVolume(BallVolumeIndex)
            if (dist(i).gt.dist_max) then
             dist_max=dist(i)
            endif
            if (dist(i).lt.dist_min) then
             dist_min=dist(i)
            endif
          enddo
         else   !This part is for grain boundary energy/curvature distributions
          do i=1, num_samplPts
           dist(i)=dist(i)/num(i)
            if (dist(i).gt.dist_max) then
             dist_max=dist(i)
            endif
            if (dist(i).lt.dist_min) then
             dist_min=dist(i)
            endif
          enddo
         endif


         !write result at each sampling point
         do i=1,num_samplPts
          vec(1)=samplPts(1,i)
          vec(2)=samplPts(2,i)
          vec(3)=samplPts(3,i)
          call VToAngles(vec,angles)
          theta_plot=angles(1)
          phi_plot=angles(2)
          write (30,*) phi_plot*(180.0/3.14159), (90.0-(theta_plot*(180.0/3.14159))), dist(i)
         enddo
         close(30) ! closes the output file

         if (errors.eq.1) then  ! Write a file with errors, when requested. Note, this does not work
          do i=1, num_samplPts
           err(i)=sqrt((dist(i)/totalArea/numDistinctGBs))/BallVolume(BallVolumeIndex)
          enddo

          do i=1,num_samplPts
           vec(1)=samplPts(1,i)
           vec(2)=samplPts(2,i)
           vec(3)=samplPts(3,i)
           call VToAngles(vec,angles)
           phi_plot=angles(1)
           theta_plot=angles(2)
           write (31,*) phi_plot*(180.0/3.14159), (90.0-(theta_plot*(180.0/3.14159))), err(i)
          enddo
          close(31) ! closes the output file

         else
          continue
         endif

         interval = (dist_max-dist_min)/10.0 !Provide some guidance for setting the limits on the plot

         !this part outputs instructions for initiating Draw_stereograms.  One case for
         !hexagonal and another for cubic.
         if (msym.eq.2) then
          write(6,*)' '
          write(6,6)'to initiate the gmt script, enter ./Draw_stereograms [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval] stereo HEX'
          write(6,20)'For example: ./Draw_stereograms 1 ',keyword(1:point),name,'_gmt 5d rainbow ',dist_min,dist_max,interval,' stereo HEX'
         else
          if (dist_type.eq.1) then
          write(6,*)' '
           write(6,6)'to initiate the gmt script, enter ./Draw_stereograms [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval]'
           write(6,22)'For example: ./Draw_stereograms 1 ',keyword(1:point),name,'_gmt 5d rainbow ',dist_min,dist_max,interval
          endif
          if (dist_type.eq.2) then
          write(6,*)' '
           write(6,6)'to initiate the gmt script, enter ./Draw_stereograms [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval]'
           write(6,22)'For example: ./Draw_stereograms_e 1 ',keyword(1:point),name,'_gmt 5d rainbow ',dist_min,dist_max,interval
          endif
          if (dist_type.eq.3) then
          write(6,*)' '
           write(6,6)'to initiate the gmt script, enter ./Draw_stereograms [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval]'
           write(6,22)'For example: ./Draw_stereograms_h 1 ',keyword(1:point),name,'_gmt 5d rainbow ',dist_min,dist_max,interval
          endif
          if (dist_type.eq.4) then
          write(6,*)' '
           write(6,6)'to initiate the gmt script, enter ./Draw_stereograms_v [Number of plots] [filename]_gmt_ 5d rainbow [min] [max] [interval]'
           write(6,22)'For example: ./Draw_stereograms_v 1 ',keyword(1:point),name,'_gmt 5d rainbow ',dist_min,dist_max,interval
          endif
         endif

 9000    continue
         if (list_out.gt.0) then
          close(32)
         endif
         !close(42)
         !write(6,*) ct1,ct2

         end






	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   
	   